#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import atexit
import logging
logger = logging.getLogger('PyAMS (scheduler')

import sys

# import interfaces
from pyams_scheduler.interfaces import SCHEDULER_HANDLER_KEY, SCHEDULER_STARTER_KEY, SCHEDULER_NAME
from pyams_utils.interfaces import PYAMS_APPLICATION_SETTINGS_KEY, PYAMS_APPLICATION_DEFAULT_NAME
from pyramid.interfaces import IApplicationCreated
from zope.interface.interfaces import ComponentLookupError

# import packages
from pyams_scheduler.process import SchedulerProcess, SchedulerMessageHandler
from pyams_utils.registry import set_local_registry
from pyams_utils.zodb import get_connection_from_settings
from pyams_zmq.process import process_exit_func
from pyramid.events import subscriber
from pyramid.settings import asbool
from zope.component.globalregistry import getGlobalSiteManager


def include_package(config):
    """Pyramid package include"""

    # add translations
    config.add_translation_dirs('pyams_scheduler:locales')

    # load registry components
    try:
        import pyams_zmi
    except ImportError:
        config.scan(ignore='pyams_scheduler.zmi')
    else:
        config.scan()


@subscriber(IApplicationCreated)
def handle_new_application(event):
    """Start scheduler process when application is created"""

    # Check for upgrade mode
    if sys.argv[0].endswith('pyams_upgrade'):
        return

    registry = getGlobalSiteManager()
    settings = registry.settings
    start_handler = asbool(settings.get(SCHEDULER_STARTER_KEY, False))
    if start_handler:
        # get database connection
        connection = get_connection_from_settings(settings)
        root = connection.root()
        # get application
        application_name = settings.get(PYAMS_APPLICATION_SETTINGS_KEY, PYAMS_APPLICATION_DEFAULT_NAME)
        application = root.get(application_name)
        process = None
        if application is not None:
            sm = application.getSiteManager()
            set_local_registry(sm)
            try:
                scheduler_util = sm.get(SCHEDULER_NAME)
                try:
                    zodb_name = scheduler_util.zodb_name
                except ComponentLookupError:
                    pass
                else:
                    # create scheduler process
                    process = SchedulerProcess(settings.get(SCHEDULER_HANDLER_KEY, '127.0.0.1:5555'),
                                               SchedulerMessageHandler, registry)
                    # load tasks
                    for task in scheduler_util.values():
                        trigger = task.get_trigger(registry)
                        logger.debug("Adding scheduler job for task '{0.name}'".format(task))
                        process.scheduler.add_job(task, trigger,
                                                  id=str(task.internal_id),
                                                  name=task.name,
                                                  kwargs={'zodb_name': zodb_name,
                                                          'registry': registry})
                    # start process
                    logger.info("Starting tasks scheduler {0!r}...".format(process))
                    process.start()
                    if process.is_alive():
                        atexit.register(process_exit_func, process=process)
                        logger.info("Started tasks scheduler with PID {0}.".format(process.pid))
            finally:
                if process and not process.is_alive():
                    process.terminate()
                    process.join()
                set_local_registry(None)
